/*******************************************************************************
SBMutFixTeleportersURL: This mutator runs once after a map is loaded and checks
all Teleporters for a URL that points to another level (technically:
Teleporter.URL contains a '#'). If such a teleporter is found, it will be
disabled. Optionally a MonsterEnd is spawned at that location.

2016-08-21, SeriousBarbie at googlemail dot com
*******************************************************************************/

class SBMutFixTeleportersURL expands Mutator config(SBMutFixTeleportersURL);

#exec OBJ LOAD File="..\System\Monsterhunt.u"

var config bool		bFixTeleportersWithURL;
var config bool		bLogActions;
var config bool		bSpawnMHEnd;
var config bool		bMHEndVisible;



function bool FixTeleporterWithURL(Teleporter T) {
/*******************************************************************************
Returns TRUE if *T* was disabled, because it has a travelling URL. A MonsterEnd
is also spawned at *T*'s location.
If *bForce*, the teleporter is treated as if it has a travelling URL.

If URL contains the map itself, the teleporter is treated also.
*******************************************************************************/
local Actor MHEnd;
local int i;
local string s;

	i = InStr(T.URL, "#");
	if (i < 0)
		Return false;

	// if (Caps(MapFileName) == Caps(Left(T.URL, i))) // teleports to the same map
	//	Return false;
	// URL=LevelName#TeleporterName: Teleports to a different level. Found in MH-FissionSmelter, MH-NP04Hyperion, MH-DKdark, MH-Decks (same level).
	if (bLogActions)
	{
		if (bSpawnMHEnd)
		{
			s = " and spawning a";
			if (bMHEndVisible)
				s = s @ "visible";
			else
				s = s @ "hidden";
			s = s @ "MonsterEnd";
		}
		Log(self @ "FixTeleporterWithURL: " @ T @ "has travelling URL '" $ T.URL $ "', disabling teleporter" $ s);
	}
	T.URL = "";
	T.SetCollision(False, False, False);// don't replace to avoid error messages "destination not found" ingame
	T.bHidden = True;
	T.Disable('Touch');
	if (bSpawnMHEnd) // put a MHEnd also at the same place:
	{
		MHEnd = Spawn(class'Monsterhunt.MonsterEnd', , , T.location);
		if (MHEnd != None)
			MHEnd.SetCollisionSize(FMax(t.CollisionRadius, MHEnd.CollisionRadius), FMax(t.CollisionHeight, MHEnd.CollisionHeight));
	}
	Return True;
}



function int FixTeleportersWithURL() {
/*******************************************************************************
Disables all Teleporters with a travelling URL and spawns a MHEnd there.
*******************************************************************************/
local Teleporter T;
local int result;
local bool bMonsterEndDefaultbHidden;

	if (bSpawnMHEnd)
	{
		bMonsterEndDefaultbHidden = class'Monsterhunt.MonsterEnd'.default.bHidden;
		class'Monsterhunt.MonsterEnd'.default.bHidden = ! bMHEndVisible;
	}

	foreach AllActors(class 'Teleporter', T)
		if (FixTeleporterWithURL(T))
			result++;

	// reset default value:
	if (bSpawnMHEnd)
		class'Monsterhunt.MonsterEnd'.default.bHidden = bMonsterEndDefaultbHidden;

	return result;
}


auto state Ready {
begin:
	if(bFixTeleportersWithURL)
		Log(self @ "Auto State Ready.Begin: number of fixed Teleporters with URL:" @ FixTeleportersWithURL());
	else
		Log(self @ "Auto State Ready.Begin: Mutator via INI settings disabled, exiting...");
}


defaultproperties {
	bFixTeleportersWithURL=true
	bLogActions=true
	bSpawnMHEnd=true
	bMHEndVisible=true
}

